---
title: vue-cli 脚手架
date: 2020-08-12 10:57:07
tags: vue
---

# 脚手架的概念

 可以帮助我们快速搭建项目，启动服务，减少这些重复性的工作。并且脚手架中包含了项目中常用的包，比如：webpack(babel、loader)，项目更加规范化，维护起来更加方便。

# 安装

 \$ npm install @vue/cli -g

- 安装完毕，通过 `vue -V` 检查脚手架是否可用。

# 创建项目

1. 命令
   cd

- \$ vue create "项目名称"

  **注意：创建的时候，会让你选择一些配置项。**

2. 项目目录结构

- package.json
  ```
  "scripts": {
    "serve": "vue-cli-service serve", // 类似于 webpack-dev-server，调试阶段实时打包编译，只不过代码都是放在内存中了。
    "build": "vue-cli-service build" // 最终的项目构建使用build，类似之前的webpack，在当前项目下生成 dist 目录。
  },
  ```
- src 目录

   是项目的源代码文件，所有代码，组件/js/css 等都放在该目录下

- main.js 文件

   是整个项目的入口文件，项目启动就会加载该文件。当项目启动时，webpack 也会自动对该文件进行打包，放到内存中。

- App.vue 文件

   根组件的模板内容，项目启动之后，首先要加载的模板内容。.vue 是一个单文件组件，每个组件都有 `<template>，<script>，<style>` 三部分构成。

- views 目录

   用来保存项目当中的 **页面级** 组件，通过路由 router 来展示页面，称为页面级组件。

- components 目录

   局部组件放到该目录下，不需要通过路由 router 来展示的页面，称为局部组件。局部组件一般都是多个页面都需要用到的某一部分内容的组件。

- assets 目录

   是静态资源目录，可以保存图片，css 等静态资源文件；它的内容既可以在 js 中使用 import 模块化的方式导入使用，也可以在页面中使用 link, script src 等引入；

- public 目录
   是静态资源目录，可以保存图片，css 等静态资源文件；只能在页面中使用 link, script src 等引入；不能使用 import 导入。

# 启动项目

 \$ npm run serve / build

# 问题

1. vue-cli 3.0 版本之后，webpack 的配置文件 `webpack.config.js` 默认隐藏了，如果需要配置 webpack，可以在项目根目录下，创建一个 `vue.config.js` 文件，这个文件中配置和 webpack 的配置是兼容的，当启动项目的时候，`vue.config.js` 文件中的配置，会自动合并到 `webpack.config.js` 中去。
2. 只要是修改了 `vue.config.js` 文件，必须重启服务才能生效。

# 插件

 安装 vuter 插件

# axios 发送请求的包

 这个包仅仅是发送请求的，体积更小，更加轻量，如果不需要操作 DOM，只是向接口发送请求，通常都会用这个包。如果项目中必须使用 jquery，那就不用装这个包了。

1. 安装 `npm install axios -S`
2. 因为各个组件都要使用，因此在 main.js 文件中导入这个包
3. 全家桶(技术栈：vue+vue-router+axios+vuex+bootstrap)
4. 配置

   ```
   import axios from "axios";
   Vue.prototype.axios = axios;
   ```

# 安装 json-server 包

 在本地启动一个服务，专门返回 json 数据的。json 数据需要自己准备.

1. `npm install json-server -g`
2. 在一个 xxx.json 文件中，准备 json 数据
3. 利用 `json-server` 打开 xxx.json 文件 `json-server xxx.json --port 3000 --host 192.168.5.252`
4. 其他地方就可以访问这个接口 `http://localhost:3000/users`

# 同步加载组件和异步加载组件

- 同步加载：当项目启动的时候，所有的组件不管你用不用，webpack 都已经帮你加载并打包好了；
- 异步加载：当项目启动的时候，所有的组件不是一块加载的，只加载项目启动之后要展示的组件，其他组件当你访问路由的时候，再实现加载；(懒加载)；

- 同步组件在项目启动时，将所有同步组件的 js 代码都打包到了 app.js 中；而异步组件在项目启动时，并没有直接打包，而是访问路由要加载组件了，才实现打包，异步组件分开打包，可以自定义打包名称 `/* webpackChunkName: "about" */`，并且异步组件打包之后的内容会被缓存起来，防止每次访问路由都重复打包。
