---
title: webpack
date: 2020-09-12 13:57:07
tags:
---

# 一、02-wp-base 项目配置

1. 新建 index.html 和 main.js
2. 使用命令 **\$`npm init -y`** 进行项目的初始化
3. 安装 **\$`npm install jquery`**这个包

# 二、通过 webpack 的配置文件实现打包

1. `webpack.config.js`配置文件，文件名称不能修改，是固定的
2. 配置文件的内容：

```
const path = require("path");

// 通过 Node 的语法向外暴露一个打包配置对象
module.exports = {
  // entry入口文件，表示要打包哪个文件
  entry: "./src/main.js",
  // output是输出目录，打包好的文件放在哪。
  output: {
    filename: "bundle.js",
    // __dirname 获取项目根目录，此配置会在根目录下生成一个 dist 目录
    path: path.resolve(__dirname, "dist"),
  },
};
```

3. 直接使用命令 `webpack` 执行打包过程，webpack 会自动找到这个配置文件。

# 三、使用 webpack-dev-server 工具

 该工具可以和 webpack 工具结合使用，目的就是实现实时打包，不用重复打包，提高开发效率。这个工具和 webpack 是两个工具，它只不过是对原生的 webpack 封装，启动一个本地服务，时刻监听代码是否发生修改，只要修改，该工具会自动编译。

- dependencies 和 devDependencies 区别
- dependencies 是生产环境(项目的正式运行环境)中仍然要使用的包，比如 jquery, bootstrap
- devDependencies 是开发环境中才使用(调试，方便)，生产环境不使用的包，比如：webpack，webpack-cli，webpack-dev-server。
- 使用 `npm install jquery -S/npm install jquery` 直接保存到生产依赖中；
- 使用 `npm install jquery -D` 直接保存到开发依赖中；

1. 在当前项目目录下执行安装 **\$`npm install webpack-dev-server -D`**
2. 在 `package.json` 中配置一条启动服务的命令

```
"scripts": {
  "test": "echo \"Error: no test specified\" && exit 1",
  --contentBase src这个配置就是指定这个服务要加载的静态目录的名称
  "dev": "webpack-dev-server --contentBase src"
},
```

3. 在当前项目目录下执行安装 **\$`npm install webpack webpack-cli -D`** 因为 webpack-dev-server 这个工具需要依赖 webpack，因此他们三个要装在一起
4. 可以通过 `http://localhost:8080` 访问页面了
5. 因为 webpack-dev-server 这个工具 是实时打包，实时编译，为了提高编译的速度，它默认将打包好的文件，放在了计算机的内存中，这样的话读写速度更快。所以 `index.html` 中就不能再引入本地的 `bundle.js` 文件了。因为该工具默认是去内存中查找 `bundle.js`
6. 继续配置启动命令，当启动服务的时候，自动打开浏览器(--open)，并指定端口(--port 3000)，然后重启服务

```
"dev": "webpack-dev-server --contentBase src --open --port 3000"
```

### 注意：webpack-dev-server 在内存中生成的文件，只是帮我们快速调试，开发方便。如果所有代码已经调试完毕，再通过 webpack 手动打包 文件 到 项目目录下。

# 四、使用插件 html-webpack-plugin

 由于 `index.html` 是在本地磁盘中，而 `bundle.js` 是在内存中存储的，因为我们需要时刻注意 `index.html` 引入 js 的路径问题。所以，这个插件可以帮助我们把 `index.html` 也在内存中生成一份，和 内存中的 `bundle.js` 放在一起。

1. 在当前项目目录下执行安装 **\$`npm install html-webpack-plugin -D`**
2. 在 `webpack.config.js` 文件中配置这个插件:

```
var HtmlWebpackPlugin = require('html-webpack-plugin');
plugins: [
  new HtmlWebpackPlugin({
    template: path.resolve(__dirname, "src/index.html"), // 指定这个插件要拷贝的模板是哪一个文件
    filename: "index.html", // 指定内存中的模板名称叫什么
  }),
],
```

3. 接下来，可以去修改 `index.html` 和 `main.js` 了，只要一修改，webpack-dev-server 就会自动编译打包 两个文件到内存中

# 五、使用 loader 处理 css 文件

 因为 webpack 默认只能打包 js 文件，其他的文件，比如 css, less, scss, 图片文件等都需要通过 loader 处理。

1. 在当前项目目录下执行安装 **\$`npm install css-loader style-loader -D`**
2. 修改 `webpack.config.js`文件：

```
module: {
  // module 属性是用来配置所有的第三方loader加载器的属性。
  rules: [
    {
      // test就是检测文件类型是否是css文件，如果是就是用use这些loader处理文件
      test: /\.css$/,
      use: ["style-loader", "css-loader"],
    },
  ],
},
```

# 六、使用 loader 处理 less 文件

1. 在当前项目目录下执行安装 **\$`npm install --save-dev less-loader less`**
2. 修改 `webpack.config.js`文件：

```
module: {
  // module 属性是用来配置所有的第三方loader加载器的属性。
  rules: [
    {
      test: /\.less$/,
      // 因为 less 语法最终也会被转化为 css，因此也需要使用 "style-loader"和"css-loader"
      use: ["style-loader", "css-loader", "less-loader"],
    },
  ],
},
```

# 七、使用 loader 处理 sass/scss 文件

1. 在当前项目目录下执行安装 **\$`npm install sass-loader node-sass -D`**
2. 修改 `webpack.config.js`文件：

```
module: {
  // module 属性是用来配置所有的第三方loader加载器的属性。
  rules: [
    {
      test: /\.scss$/,
      use: [{
          loader: "style-loader" // 将 JS 字符串生成为 style 节点
      }, {
          loader: "css-loader" // 将 CSS 转化成 CommonJS 模块
      }, {
          loader: "sass-loader" // 将 Sass 编译成 CSS
      }]
    }
  ],
},
```

# 八、使用 loader 处理 图片 文件

1. 在当前项目目录下执行安装 **\$`npm install url-loader file-loader -D`**
2. 修改 `webpack.config.js`文件：

```
module: {
  // module 属性是用来配置所有的第三方loader加载器的属性。
  rules: [
    {
      test: /\.(png|jpg|gif)$/,
      use: [ "url-loader" ],
    },
  ],
},
```

# 九、使用 babel-loader 处理 js 高级语法

 webpack 已经可以处理一部分 js 中的高级语法，转换成低级语法，让浏览器识别，但是仍然有一部分 js 的高级语法需要借助于 loader 加载器处理

1. 安装 `npm install babel-loader @babel/core @babel/preset-env @babel/plugin-transform-runtime @babel/plugin-proposal-class-properties -D`
2. 安装 `npm install @babel/runtime` 到生产依赖

3. @babel/core 内部包含了转换代码的 API 函数，@babel/preset-env 它是指定了语法转换规则，@babel/plugin-transform-runtime 减少编译后的代码体积

4. 修改 `webpack.config.js` 配置文件：

```
{
  test: /\.js$/,
  // exclude: 排除 node_modules，这里面的js文件不需要处理
  exclude: /(node_modules|bower_components)/,
  use: {
    loader: "babel-loader",
    options: {
      // preset-env 这里 env 是一种语法转换规则，preset就是预设的意思。
      presets: ["@babel/preset-env"],
      plugins: ["@babel/transform-runtime", @babel/plugin-proposal-class-properties],
    },
  },
},
```
